'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.formattedTimestamp = formattedTimestamp;
exports.minDurationMsPerTimelinePx = minDurationMsPerTimelinePx;
exports.maxDurationMsPerTimelinePx = maxDurationMsPerTimelinePx;
exports.initialDurationMsPerTimelinePx = initialDurationMsPerTimelinePx;
exports.getTimeScale = getTimeScale;
exports.findOptimalDurationFit = findOptimalDurationFit;

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _lodash = require('lodash');

var _d3Scale = require('d3-scale');

var _timeline = require('../constants/timeline');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function formattedTimestamp(timestamp) {
  var momentTimestamp = timestamp ? (0, _moment2.default)(timestamp) : (0, _moment2.default)();
  return momentTimestamp.startOf('second').utc().format();
}

function availableTimelineDurationMs(earliestTimestamp) {
  var earliestMomentTimestamp = (0, _moment2.default)(earliestTimestamp);
  var currentMomentTimestamp = (0, _moment2.default)(formattedTimestamp());
  return currentMomentTimestamp.diff(earliestMomentTimestamp);
}

// The most granular zoom is 2px per second, probably we don't want any more granular than that.
function minDurationMsPerTimelinePx() {
  return _moment2.default.duration(500, 'milliseconds').asMilliseconds();
}

// Maximum level we can zoom out is such that the available range takes 400px. The 3 days
// per pixel upper bound on that scale is to prevent ugly rendering in extreme cases.
function maxDurationMsPerTimelinePx(earliestTimestamp) {
  var durationMsLowerBound = minDurationMsPerTimelinePx();
  var durationMsUpperBound = _moment2.default.duration(3, 'days').asMilliseconds();
  var durationMs = availableTimelineDurationMs(earliestTimestamp) / 400.0;
  return (0, _lodash.clamp)(durationMs, durationMsLowerBound, durationMsUpperBound);
}

// The initial zoom level is set to be 10% of the max zoom out level capped at 1px per minute,
// with the assumption that if we have a long recorded history, we're in most cases by
// default going to be interested in what happened in last couple of hours or so.
function initialDurationMsPerTimelinePx(earliestTimestamp) {
  var durationMsLowerBound = minDurationMsPerTimelinePx();
  var durationMsUpperBound = _moment2.default.duration(1, 'minute').asMilliseconds();
  var durationMs = maxDurationMsPerTimelinePx(earliestTimestamp) * 0.1;
  return (0, _lodash.clamp)(durationMs, durationMsLowerBound, durationMsUpperBound);
}

function getTimeScale(_ref) {
  var focusedTimestamp = _ref.focusedTimestamp,
      durationMsPerPixel = _ref.durationMsPerPixel;

  var startDate = (0, _moment2.default)(focusedTimestamp).subtract(durationMsPerPixel);
  var endDate = (0, _moment2.default)(focusedTimestamp).add(durationMsPerPixel);
  return (0, _d3Scale.scaleUtc)().domain([startDate, endDate]).range([-1, 1]);
}

function findOptimalDurationFit(periodIntervals, period, _ref2) {
  var durationMsPerPixel = _ref2.durationMsPerPixel;

  var minimalDurationMs = durationMsPerPixel * 1.1 * _timeline.MIN_TICK_SPACING_PX;
  return (0, _lodash.find)(periodIntervals, function (p) {
    return _moment2.default.duration(p, period).asMilliseconds() >= minimalDurationMs;
  });
}